(*|  0:03 17/05/1997 *)
PROGRAM Year2000;

USES
  Crt,Dos;

CONST
  Days : array[0..7] of string[3] =
             ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat','---');
  Months : array[1..12] of string[3] =
             ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
              'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');

TYPE
  TimeType= RECORD
              Hours,Mins,Secs,Tenths: Byte;
            END;

  TimeStrType = String[5];

  DateType= RECORD
              Day,Month,Year: Integer;
            END;

  DateStrType = String[8];

VAR
  Done,Modified: Boolean;
  RTClock,OldClock,DosClock,StartClock: TimeType;
  RTDate,DosDate,StartDate: DateType;
  DateNow: DateType;
  RTDayNum,DosDayNum,StartDayNum: Integer;

PROCEDURE Get_Time(Var Time: TimeType);
{Get the real time clock time from MSDOS}

VAR Regs: Registers;

BEGIN
  WITH Time,Regs DO BEGIN
    AX:= $2C00;
    Msdos(Regs);
    Hours:= CH;
    Mins:= CL;
    Secs:= DH;
    Tenths:= DL DIV 10;     {calculate 10ths from hundredths/10}
  END;
END;  {Get_Time}

PROCEDURE Set_Time(Time: TimeType);
{Set the real time clock time in MSDOS}

VAR Regs: Registers;

BEGIN
  WITH Time,Regs DO BEGIN
    CX:= (256*Hours)+Mins;
    DX:= (256*Secs);
    AX:= $2D00;
    Msdos(Regs);
  END;
END;  {Set_Time}

FUNCTION BCDBin(BCDVal: Integer): Integer;

BEGIN
  BCDBin:=(BCDVal AND $0F) + 10*((BCDVal AND $F0) Shr 4);
END;  { BCDBin }

FUNCTION BinBCD(BinVal: Integer): Integer;

BEGIN
  BinBCD:=BinVal MOD 10 + (BinVal DIV 10) Shl 4;
END;  { BinBCD }

PROCEDURE Get_RealTime(Var Time: TimeType);
{Get the real time from battery clock}

VAR Regs: Registers;

BEGIN
  WITH Time,Regs DO BEGIN
    AX:= $0200;
    INTR($1A,Regs);
    Hours:= BCDBin(CH);
    Mins:= BCDBin(CL);
    Secs:= BCDBin(DH);
    Tenths:= 0;
  END;
END;  {Get_RealTime}

PROCEDURE Set_RealTime(Time: TimeType);
{Set the real time clock time in MSDOS}

VAR Regs: Registers;

BEGIN
  WITH Time,Regs DO BEGIN
    CX:= (BinBCD(Hours) * 256)+BinBCD(Mins);
    DX:= (BinBCD(Secs) * 256) + 0;
    AX:= $0300;
    INTR($1A,Regs);
  END;
END;  {Set_RealTime}

FUNCTION IntToString(Num, Width : integer) : String;
{ Changes an integer into a string }

VAR
  TempString : String;
  I: Integer;

BEGIN
  Str(Num:Width, TempString);
  I := 1;
  WHILE TempString[I] = ' ' DO BEGIN
    TempString[I]:= '0';
    INC(I);
  END;
  IntToString := TempString;
END; { IntToString }

FUNCTION Today : Integer;
{ Gets day from DOS and returns it as an integer }

VAR Regs : Registers;         { Registers used for BIOS and DOS function calls }

BEGIN
  WITH Regs DO
  BEGIN
    AX := $2A00;
    MsDos(Regs);
    Today := Lo(AX);
  END; { with }
END; { Today }

PROCEDURE GetDate(Var Date: DateType; Var DayNum: Integer);

VAR Regs: Registers;

BEGIN
  WITH Regs,Date DO BEGIN
    AX:=$2A00;
    Msdos(Regs);
    Day:=DL;
    Month:=DH;
    Year:=CX;
    DayNum:= AL;
  END;
END; { GetDate }

PROCEDURE SetDate(Date: DateType);

VAR Regs: Registers;

BEGIN
  WITH Regs,Date DO BEGIN
    AX:=$2A00;
    Msdos(Regs);
    DX:=(256*Month)+Day;
    CX:=Year;        {Set by read operation}
    AX:=$2B00;
    Msdos(Regs);
  END;
END; { SetDate }

PROCEDURE GetRealDate(Var Date: DateType; Var DayNum: Integer);

VAR Regs: Registers;

BEGIN
  WITH Regs,Date DO BEGIN
    AX:=$0400;
    INTR($1A,Regs);
    Day:=BCDBin(DL);
    Month:=BCDBin(DH);
    Year:=(BCDBin(CH) * 100) + BCDBin(CL);
    DayNum:= 7;  {not supported}
  END;
END; { GetRealDate }

PROCEDURE SetRealDate(Date: DateType);

VAR Regs: Registers;

BEGIN
  WITH Regs,Date DO BEGIN
    AX:=$0400;
    INTR($1A, Regs);
    DH:=BinBCD(Month);
    DL:=BinBCD(Day);
    CX:=Year;        {Set by read operation}
    AX:=$0500;
    INTR($1A,Regs);
  END;
END; { SetRealDate }

PROCEDURE ShowTime(Time: TimeType);
BEGIN
  WITH Time DO BEGIN
    Write(IntToString(Hours,2) + ':' + IntToString(Mins,2) + ':' +
          IntToString(Secs,2));
  END;
END;

PROCEDURE WriteDate(Date: DateType; DayNum: Integer);

BEGIN
  WITH Date DO BEGIN
    Write(Days[DayNum] + ', ' + IntToString(Day,2) + ' ' +
          Months[Month] + ' ' + IntToString(Year,4));
  END; { with }
END; { DateString }

PROCEDURE DoIt(VAR Done: Boolean);
BEGIN
  Get_Time(DosClock);
  IF DosClock.Secs <> OldClock.Secs THEN BEGIN
    OldClock := DosClock;
    Get_RealTime(RTClock);
    GetDate(DosDate,DosDayNum);
    GetRealDate(RTDate,RTDayNum);
    GotoXY(10,3);
    ShowTime(OldClock);
    GotoXY(10,5);
    ShowTime(RTClock);

    GotoXY(20,3);
    WriteDate(DosDate,DosDayNum);
    ClrEol;
    GotoXY(20,5);
    WriteDate(RTDate,RTDayNum);
    ClrEol;
  END;
  IF KeyPressed THEN CASE UpCase(ReadKey) OF
    #$1B : Done := True;
    '2'  : WITH DosClock,DosDate DO BEGIN
             Hours:=23;
             Mins:=59;
             Secs:=50;
             Set_Time(DosClock);
             Day:=31;
             Month:=12;
             Year:=1999;
             SetDate(DosDate);
             Modified := True;
           END;
     'R' : BEGIN
             GetDate(DosDate,DosDayNum);
             SetDate(DosDate);
           END;
  END;
END;

BEGIN
  ClrScr;
  Write('Year 2000 Test Program by B Whitnall, May 1997');
  GotoXY(1,3);
  Write('DOS:');
  GotoXY(1,5);
  Write('RT Clock');
  GotoXY(1,7);
  Write('Commands : 2, R, <ESC>');
  Modified := False;
  Get_Time(StartClock);
  GetDate(StartDate,StartDayNum);
  REPEAT
    DoIt(Done);
  UNTIL Done;
  IF Modified THEN BEGIN
    Set_Time(StartClock);
    SetDate(StartDate);
    GotoXY(1,7);
    ClrEol;
    GotoXY(10,7);
    ShowTime(StartClock);
    GotoXY(20,7);
    WriteDate(StartDate, StartDayNum);
  END;
  GotoXY(1,8);
END.
